@@ -1,15 +1,16 @@
-import numpy as np
-from astropy.io import fits
-import oda_api.api
 import copy
-import matplotlib
-from astropy import units as u
-from astroquery.simbad import Simbad
+
 import astropy.wcs as wcs
+import matplotlib
 import matplotlib.pylab as plt
-from astropy.coordinates import SkyCoord
-from astropy import table
+import numpy as np
+import oda_api.api
 import requests
+from astropy import table
+from astropy import units as u
+from astropy.coordinates import SkyCoord
+from astropy.io import fits
+from astroquery.simbad import Simbad
 from autologging import logged
 
 __author__ = "Carlo Ferrigno"
@@ -17,7 +18,8 @@ __author__ = "Carlo Ferrigno"
 __all__ = ['INTEGRALwrapper']
 
 # Hopefully, this is stable enough
-oda_public_host = 'https://www.astro.unige.ch/cdci/astrooda/dispatch-data'
+oda_public_host = 'https://www.astro.unige.ch/mmoda/dispatch-data'
+legacy_oda_public_host = 'https://www.astro.unige.ch/cdci/astrooda/dispatch-data'
 
 @logged
 class INTEGRALwrapper(object):
@@ -26,11 +28,13 @@ class INTEGRALwrapper(object):
     # - product is compatible with oda_api
     # - the list of science windows can be ordered
 
-    def __init__(self, host_type='production'):
+    def __init__(self, host_type='production', token='', integral_data_rights='public'):
 
         self.product = 'None'
+        self.token = token
+        self.integral_data_rights = integral_data_rights
 
-        instrument = 'isgri'
+        instrument = 'mock'
         # if('jemx' in product):
         #     instrument='jemx'
 
@@ -39,19 +43,22 @@ class INTEGRALwrapper(object):
         if host_type == 'staging-1-3':
             host = 'http://in.internal.odahub.io/staging-1-3/dispatcher'
 
-        if host_type == 'staging' or host_type == 'staging-1-2':
+        if host_type == 'staging-1-2':
             host = 'http://cdcihn.isdc.unige.ch/staging-1.2/dispatcher'
 
+        if host_type == 'staging':
+            host = 'http://dispatcher.staging.internal.odahub.io'
+
         if host_type == 'test':
             host = 'http://cdciweb01.isdc.unige.ch:8084/astrooda'
 
         try:
             self.disp = oda_api.api.DispatcherAPI(url=host)
-            self.disp.get_instrument_description(instrument)
+            #self.disp.get_instrument_description(instrument)
         except:
             try:
                 self.disp = oda_api.api.DispatcherAPI(url=host)
-                self.disp.get_instrument_description(instrument)
+                #self.disp.get_instrument_description(instrument)
             except:
                 raise ConnectionError
 
@@ -98,50 +105,66 @@ class INTEGRALwrapper(object):
                 if n_poll == 0:
                     self.__log.info("At call %d we elaborate %d scw" % (n_call, len(scw_list)))
                     tot_num += len(scw_list)
-                #     silent = False
-                # else:
-                #     if n_poll % 10 == 0:
-                #         silent = False
-                #     else:
-                #         silent = True
 
                 ys = "%06d" % n_call
 
                 if ys not in disp_by_call:
                     disp_by_call[ys] = oda_api.api.DispatcherAPI(url=self.disp.url, wait=False)
+                    #This is just a submission
 
                 _disp = disp_by_call[ys]
-                #print(type(_disp))
                 data = data_by_call.get(ys, None)
 
-                if data is None or not _disp.is_failed():
+                self.__log.debug(f'\n\n\nn_call {n_call}\nData {data}\nIs failed { _disp.is_failed}')
+                if data is None and not _disp.is_failed:
+                    revs = get_revs(scw_list)
+                    # Force 0SA10.2 for ISGRI before rev. 1626
+                    self.__log.debug(f"min rev {revs.min()}")
+                    # if revs.min() < 1626 and 'isgri' in arguments['product']:
+                    #     self.__log.debug("Forced OSA10.2 instead of OSA11")
+                    #     self.__log.info("Forced OSA10.2 instead of OSA11")
+                    #     local_arguments['osa_version'] = 'OSA10.2'
+                    # else:
+                    local_arguments['osa_version'] = arguments['osa_version']
+
+                    #Add Token to arguments if not present
+                    if self.token != '' and 'token' not in arguments:
+                        local_arguments.update({'token': self.token})
+                        self.__log.debug("Inserted token in local arguments")
+                    else:
+                        self.__log.debug("token present in  input arguments or not defined")
+                    #Add integral data rights to arguments if not present
+                    if 'integral_data_rights' not in arguments:
+                        local_arguments.update({'integral_data_rights': self.integral_data_rights})
+                        self.__log.debug("Inserted integral_data_rights in local arguments")
+                    else:
+                        self.__log.debug("integral_data_rights present in  input arguments")
+
+
+                    scw_list_str = ",".join([s for s in sorted(set(scw_list))])
+                    self.__log.debug(f"Is submitted: {_disp.is_submitted}")
                     if not _disp.is_submitted:
-                        revs = get_revs(scw_list)
-                        # Force 0SA10.2 for ISGRI before rev. 1626
-                        if revs.min() < 1626 and 'isgri' in arguments['product']:
-                            local_arguments['osa_version'] = 'OSA10.2'
-                        else:
-                            local_arguments['osa_version'] = arguments['osa_version']
-                        scw_list_str = ",".join([s for s in sorted(set(scw_list))])
                         data = _disp.get_product(scw_list=scw_list_str, **local_arguments)
                     else:
                         _disp.poll()
 
                     data_by_call[ys] = data
-
+                    self.__log.debug("Is complete", _disp.is_complete)
                     if not _disp.is_complete:
                         continue
                     else:
-                        data_by_call[ys] = _disp.get_product(scw_list=scw_list_str, **arguments)
-
+                        data_by_call[ys] = _disp.get_product(scw_list=scw_list_str, **local_arguments)
 
+            self.__log.debug(f'n_poll: {n_poll} ')
             n_complete = len([call for call, _disp in disp_by_call.items() if _disp.is_complete])
-            self.__log.debug(f"complete {n_complete} / {len(disp_by_call)}")
+            self.__log.debug(f'n_complete: {n_complete}')
+            self.__log.debug(f"complete {float(n_complete) / len(disp_by_call)}")
             if n_complete == len(disp_by_call):
                 self.__log.info("done!")
                 break
             self.__log.debug("not done")
             n_poll += 1
+
             if not wait:
                 return None
             time.sleep(sleep_time)
@@ -152,7 +175,7 @@ class INTEGRALwrapper(object):
             self.__log.debug(kk)
             self.all_data.append(data_by_call[kk])
 
-        self.__log.debug(len(total_scw_list), tot_num)
+        self.__log.debug(f'{len(total_scw_list)}, {tot_num}')
 
         if 'spectrum' in self.product:
             return self.sum_spectra()
@@ -182,6 +205,9 @@ class INTEGRALwrapper(object):
         from astropy import table
         import oda_api.data_products
 
+        if len(self.all_data) == 1:
+            self.__log.info("We have one mosaic, returning it.")
+            return self.all_data[0]
         list_hdus = [dd.mosaic_image_0_mosaic.to_fits_hdu_list() for dd in self.all_data]
         summed_mosaic = mosaic.mosaic_list(list_hdus, pixels=projection, mock=False)
 
@@ -189,7 +215,7 @@ class INTEGRALwrapper(object):
         for dd in self.all_data:
             tab = dd.dispatcher_catalog_1.table
             if len(tab) > 0:
-                ind = np.logical_and( tab['significance'] >= detection_threshold, tab['ISGRI_FLAG'] > 0)
+                ind = np.logical_and(tab['significance'] >= detection_threshold, tab['ISGRI_FLAG'] > 0)
                 if np.sum(ind) > 0:
                     sources.append(tab[ind])
 
@@ -198,7 +224,12 @@ class INTEGRALwrapper(object):
             ss.replace_column('ERR_RAD', ss.columns['ERR_RAD'].astype(np.float64))
 
         # We stack the table but take just the first occurrence of each source
-        stacked_table_known = table.unique(table.vstack(sources, join_type='exact'), keys='src_names')
+        if len(sources) > 0:
+            stacked_table_known = table.unique(table.vstack(sources, join_type='exact'), keys='src_names')
+        else:
+            stacked_table_known = None
+            self.__log.warning("No sources satisfy the criterion for filtering")
+
 
         new_sources_list = [dd.dispatcher_catalog_1.table[dd.dispatcher_catalog_1.table['ISGRI_FLAG'] == 0]
                             for dd in self.all_data]
@@ -219,57 +250,81 @@ class INTEGRALwrapper(object):
 
         self.__log.debug(new_sources_list)
         self.__log.debug(stacked_table_known)
-        stacked_table = table.unique(table.vstack(new_sources_list + [stacked_table_known], join_type='exact'),
+        if len(new_sources_list) > 0 and stacked_table_known is not None:
+            stacked_table = table.unique(table.vstack(new_sources_list + [stacked_table_known], join_type='exact'),
                                      keys='src_names')
-
-        idx_f, idx_s = find_duplicates(stacked_table)
-
-        # TODO tests with duplicates
-
-        if idx_f is None:
-            self.__log.info("No duplicates in final catalog")
+        elif len(new_sources_list) == 0 and stacked_table_known is not None:
+            stacked_table = stacked_table_known
         else:
-            self.__log.info("Removing %d duplicates" % len(idx_f))
-            stacked_table.remove_rows(idx_s)
+            stacked_table = None
+
+        if stacked_table is not None:
+            idx_f, idx_s = find_duplicates(stacked_table)
+
+            if idx_f is None:
+                self.__log.info("No duplicates in final catalog")
+            else:
+                self.__log.info("Removing %d duplicates" % len(idx_f))
+                stacked_table.remove_rows(idx_s)
+
+            summed_data.dispatcher_catalog_1.table = stacked_table
 
-        summed_data.dispatcher_catalog_1.table = stacked_table
         summed_data.mosaic_image_0_mosaic.data_unit = summed_data.mosaic_image_0_mosaic.data_unit[0:1] + \
                                                       [oda_api.data_products.NumpyDataUnit.from_fits_hdu(hh) for hh in
                                                        summed_mosaic.to_hdu_list()[1:]
                                                        if hh.header['IMATYPE'] != 'NIMAGE']
 
+        self.summed_data.dispatcher_catalog_1.table = INTEGRALwrapper.compute_fluxes(summed_data, detection_threshold)
+        INTEGRALwrapper.display_sources(summed_data.dispatcher_catalog_1.table, detection_threshold)
         self.summed_data=summed_data
-        self.compute_fluxes()
-        self.display_sources()
+
         return summed_data
 
-    def display_sources(self):
+    @staticmethod
+    def display_sources(stacked_table, min_sigma):
         out_str = 'source_name RA DEC Flux Flux_err\n-----------------------------------'
-        self.__log.info(out_str)
-
-        for ss in self.summed_data.dispatcher_catalog_1.table:
-            out_str = '%s %.3f %.3f %.4f %.4f' % (ss['src_names'], ss['ra'], ss['dec'], ss['FLUX'],
-                                                    ss['FLUX_ERR'])
-            self.__log.info(out_str)
+        INTEGRALwrapper.__log.info(out_str)
 
 
-    def compute_fluxes(self):
+        if 'FLUX' in stacked_table.keys():
+            for ss in stacked_table:
+                if ss['significance'] > min_sigma:
+                    out_str = '%s %.3f %.3f %.4f %.4f' % (ss['src_names'], ss['ra'], ss['dec'], ss['FLUX'],
+                                                        ss['FLUX_ERR'])
+                    INTEGRALwrapper.__log.info(out_str)
+        else:
+            for ss in stacked_table:
+                if ss['significance'] > min_sigma:
+                    out_str = '%s %.3f %.3f %.4f %d' % (ss['src_names'], ss['ra'], ss['dec'], ss['significance'],
+                                                        ss['FLAG'])
+                    INTEGRALwrapper.__log.info(out_str)
 
-        if not hasattr(self, 'summed_data'):
-            raise RuntimeWarning('No summed mosaic, no computation of fluxes')
+    @staticmethod
+    def compute_fluxes(summed_data, min_sigma=5, free_sigma=False, free_constant=False):
+
+        # if not hasattr(self, 'summed_data'):
+        #     raise RuntimeWarning('No summed mosaic, no computation of fluxes')
+        #
+        stacked_table = summed_data.dispatcher_catalog_1.table
+        if len(stacked_table) == 0:
+            self.__log.warning("No sources in the catalog, impossible to compute fluxes")
+            return
+
+        #Filter sources by significance
+        stacked_table = stacked_table[stacked_table['significance'] > min_sigma]
 
-        stacked_table = self.summed_data.dispatcher_catalog_1.table
         cat_for_image = INTEGRALwrapper.get_source_list_from_table(stacked_table)
         import oda_integral_wrapper.fitimage as fitimage
-        # import importlib
-        # importlib.reload(fitimage)
-        fit_image = fitimage.FitMosaicSources(self.summed_data.mosaic_image_0_mosaic.data_unit, cat_for_image)
+        import importlib
+        importlib.reload(fitimage)
+        fit_image = fitimage.FitMosaicSources(summed_data.mosaic_image_0_mosaic.data_unit, cat_for_image,
+                                              free_sigma=free_sigma, free_constant=free_constant)
         fitted_fluxes = fit_image.get_fluxes()
         stacked_table['FLUX'] = fitted_fluxes[0][1]
         stacked_table['FLUX_ERR'] = fitted_fluxes[0][2]
         stacked_table['significance'] = fitted_fluxes[0][1] / np.array(fitted_fluxes[0][2])
 
-        self.summed_data.dispatcher_catalog_1.table = stacked_table
+        return stacked_table
 
     def write_ds9_region_file(self, filename='ds9.reg', color='green', new_color='white'):
 
@@ -311,6 +366,8 @@ class INTEGRALwrapper(object):
         if not 'lc' in combined_data._p_list[0].name:
             raise ValueError('This is not a light curve and you try to stitch them')
 
+        if len(self.all_data) == 1:
+            return combined_data
         sources = self.get_sources()
 
         # gets indexes of source and lc in combined data
@@ -371,7 +428,46 @@ class INTEGRALwrapper(object):
 
         return combined_data
 
+    @staticmethod
+    def normalize_lc_ijd(combined_lc, source_name, Emin, Emax):
+        new_combined_lc = copy.deepcopy(combined_lc)
+        t, dt, y, dy = INTEGRALwrapper.get_lc(new_combined_lc, source_name)
+        ind = t < 5838.086367870370
+        from oda_integral_wrapper.get_osa10_11_factor import get_osa10_11_conversion_factor
+        f = get_osa10_11_conversion_factor(Emin, Emax)
+        y[ind] /= f
+        dy[ind] /= f
+        INTEGRALwrapper.put_lc(new_combined_lc, source_name, t, y, dy)
+        return new_combined_lc
 
+    @staticmethod
+    def put_lc(combined_lc, source_name, x, y, dy):
+        # In LC name has no "-" nor "+" ??????
+        patched_source_name = source_name.replace('-', ' ').replace('+', ' ')
+
+        hdu = None
+        j_index = -1
+        i_index = -1
+        for j, dd in enumerate(combined_lc._p_list):
+            INTEGRALwrapper.__log.debug(dd.meta_data['src_name'])
+            if dd.meta_data['src_name'] == source_name or dd.meta_data['src_name'] == patched_source_name:
+                for ii, du in enumerate(dd.data_unit):
+                    if 'LC' in du.name:
+                        hdu = du.to_fits_hdu()
+                        i_index = ii
+                        j_index = j
+
+        if hdu is None:
+            INTEGRALwrapper.__log.info('Source ' + source_name + ' not found in lc')
+            return None
+
+        hdu.data['TIME'] = x
+        hdu.data['RATE'] = y
+        hdu.data['ERROR'] = dy
+
+        combined_lc._p_list[j_index].data_unit[i_index].from_fits_hdu(hdu)
+
+        return
 
     @staticmethod
     def get_lc(combined_lc, source_name, systematic_fraction=0):
@@ -379,7 +475,6 @@ class INTEGRALwrapper(object):
         # In LC name has no "-" nor "+" ??????
         patched_source_name = source_name.replace('-', ' ').replace('+', ' ')
 
-
         hdu = None
         for j, dd in enumerate(combined_lc._p_list):
             INTEGRALwrapper.__log.debug(dd.meta_data['src_name'])
@@ -389,7 +484,7 @@ class INTEGRALwrapper(object):
                         hdu = du.to_fits_hdu()
 
         if hdu is None:
-            INTEGRALwrapper.__log.info('Source ' + source_name + ' not found in lc, not plotting')
+            INTEGRALwrapper.__log.info('Source ' + source_name + ' not found in lc')
             return None
 
         x = hdu.data['TIME']
@@ -407,7 +502,7 @@ class INTEGRALwrapper(object):
 
         #This could only be valid for ISGRI
         try:
-            dt_lc =  hdu.data['XAX_E']
+            dt_lc = hdu.data['XAX_E']
             INTEGRALwrapper.__log.debug('Get time bin directly from light curve')
         except:
             timedel = hdu.header['TIMEDEL']
@@ -420,9 +515,9 @@ class INTEGRALwrapper(object):
 
         return x[ind], dt_lc[ind], y[ind], dy[ind]
 
-
     @staticmethod
     def plot_lc(combined_lc, source_name, systematic_fraction=0, ng_sig_limit=3, find_excesses=False):
+        #if ng_sig_limit <1 does not plot range
         from scipy import stats
         x,dx,y,dy = INTEGRALwrapper.get_lc(combined_lc, source_name, systematic_fraction)
 
@@ -437,20 +532,21 @@ class INTEGRALwrapper(object):
         _ = plt.xlabel('Time [IJD]')
         _ = plt.ylabel('Rate')
 
-        ndof = len(y) - 1
-        prob_limit = stats.norm().sf(ng_sig_limit)
-        chi2_limit = stats.chi2(ndof).isf(prob_limit)
-        band_width = np.sqrt(chi2_limit / err_mean)
-        INTEGRALwrapper.__log.debug('%g %g %g %g %g %g %g' %(
-            meany, err_mean, std_dev, prob_limit, chi2_limit, band_width, ng_sig_limit))
-        _ = plt.axhspan(meany - band_width, meany + band_width, color='green', alpha=0.3,
-                        label=f'{ng_sig_limit} $\sigma_m$, {100 * systematic_fraction}% syst')
+        if ng_sig_limit >= 1:
+            ndof = len(y) - 1
+            prob_limit = stats.norm().sf(ng_sig_limit)
+            chi2_limit = stats.chi2(ndof).isf(prob_limit)
+            band_width = np.sqrt(chi2_limit / err_mean)
+            INTEGRALwrapper.__log.debug('%g %g %g %g %g %g %g' %(
+                meany, err_mean, std_dev, prob_limit, chi2_limit, band_width, ng_sig_limit))
+            _ = plt.axhspan(meany - band_width, meany + band_width, color='green', alpha=0.3,
+                            label=f'{ng_sig_limit} $\sigma_m$, {100 * systematic_fraction}% syst')
 
-        _ = plt.axhspan(meany - std_dev*ng_sig_limit, meany + std_dev*ng_sig_limit,
-                        color='cyan', alpha=0.3,
-                        label=f'{ng_sig_limit} $\sigma_d$, {100 * systematic_fraction}% syst')
+            _ = plt.axhspan(meany - std_dev*ng_sig_limit, meany + std_dev*ng_sig_limit,
+                            color='cyan', alpha=0.3,
+                            label=f'{ng_sig_limit} $\sigma_d$, {100 * systematic_fraction}% syst')
 
-        _ = plt.legend()
+            _ = plt.legend()
 
         plot_title = source_name
         _ = plt.title(plot_title)
@@ -517,10 +613,25 @@ class INTEGRALwrapper(object):
         if not include_new_sources:
             ind = [not 'NEW' in ss for ss in sources['src_names']]
             clean_sources = sources[ind]
-            INTEGRALwrapper.__log.debug(ind, sources, clean_sources)
+            INTEGRALwrapper.__log.debug(ind)
+            INTEGRALwrapper.__log.debug(sources)
+            INTEGRALwrapper.__log.debug(clean_sources)
         else:
             clean_sources = sources
 
+        unique_sources = INTEGRALwrapper.add_objects_of_interest(clean_sources, objects_of_interest,
+                                                                 flag, isgri_flag)
+
+        copied_image = copy.deepcopy(image)
+        copied_image.dispatcher_catalog_1.table = unique_sources
+
+        if update_catalog:
+            image.dispatcher_catalog_1.table = unique_sources
+
+        return copied_image.dispatcher_catalog_1.get_api_dictionary()
+
+    @staticmethod
+    def add_objects_of_interest(clean_sources, objects_of_interest, flag=1, isgri_flag=2, tolerance = 1./60.):
         for ooi in objects_of_interest:
             if isinstance(ooi, tuple):
                 ooi, t = ooi
@@ -541,6 +652,18 @@ class INTEGRALwrapper(object):
             ra = source_coord.ra.deg
             dec = source_coord.dec.deg
             INTEGRALwrapper.__log.info("RA=%g Dec=%g" % (ra, dec))
+
+            #Look for the source of interest in NEW sources by coordinates
+            for ss in clean_sources:
+                if 'NEW' in ss['src_names']:
+                    if np.abs(ra - ss['ra']) <= tolerance and np.abs(dec - ss['dec']) <= tolerance:
+                        INTEGRALwrapper.__log.info('Found ' + ooi + ' in catalog as ' + ss['src_names'])
+                        ind = clean_sources['src_names'] == ss['src_names']
+                        clean_sources['FLAG'][ind] = flag
+                        clean_sources['ISGRI_FLAG'][ind] = isgri_flag
+                        clean_sources['src_names'][ind] = ooi
+
+            #Look for the source of interest in
             ind = clean_sources['src_names'] == ooi
             if np.count_nonzero(ind) > 0:
                 INTEGRALwrapper.__log.info('Found ' + ooi + ' in catalog')
@@ -548,21 +671,19 @@ class INTEGRALwrapper(object):
                 clean_sources['ISGRI_FLAG'][ind] = isgri_flag
             else:
                 INTEGRALwrapper.__log.info('Adding ' + ooi + ' to catalog')
-                clean_sources.add_row((0, ooi, 0, ra, dec, 0, isgri_flag, flag, 1e-3, 0, 0))
+                try:
+                    INTEGRALwrapper.__log.debug('Flux is present')
+                    clean_sources.add_row((0, ooi, 0, ra, dec, 0, isgri_flag, flag, 1e-3, 0, 0))
+                except:
+                    INTEGRALwrapper.__log.debug('Flux is NOT present')
+                    clean_sources.add_row((0, ooi, 0, ra, dec, 0, isgri_flag, flag, 1e-3))
 
         unique_sources = table.unique(clean_sources, keys=['src_names'])
 
-        copied_image = copy.deepcopy(image)
-        copied_image.dispatcher_catalog_1.table = unique_sources
-
-        if update_catalog:
-            image.dispatcher_catalog_1.table = unique_sources
-
-        return copied_image.dispatcher_catalog_1.get_api_dictionary()
+        return unique_sources
 
     @staticmethod
     def sum_spectral_products(spectrum_results, source_name):
-
         d = spectrum_results[0]
 
         ID_spec = -1
@@ -581,7 +702,7 @@ class INTEGRALwrapper(object):
                 break
         INTEGRALwrapper.__log.info('Initialize with IDs for spe, arf and rmf %d %d %d' % (ID_spec, ID_arf, ID_rmf))
 
-        d = spectrum_results[0]
+        #d = spectrum_results[0]
         spec = d._p_list[ID_spec].data_unit[1].data
         arf = d._p_list[ID_arf].data_unit[1].data
         rmf = d._p_list[ID_rmf].data_unit[2].data
@@ -680,6 +801,9 @@ class INTEGRALwrapper(object):
         if not 'spectrum' in summed_data._p_list[0].meta_data['product']:
             raise ValueError('This is not a spectrum and you try to sum spectra')
 
+        if len(self.all_data) == 1:
+            return summed_data
+
         sources = self.get_sources()
         self.__log.debug(sources)
         for source in sources:
@@ -746,6 +870,23 @@ class INTEGRALwrapper(object):
 
         lc_fn = output_dir + "/%s_lc_%s_%s.fits" % (instrument, source_name.replace(' ', '_'), subcases_pattern)
         hdu = lcprod[0].data_unit[ind_extension].to_fits_hdu()
+        timedel = hdu.header['TIMEDEL']
+        timepixr = hdu.header['TIMEPIXR']
+
+        dt = timedel * timepixr
+
+        hdu.header['TSTART'] = hdu.data['TIME'][0] - dt
+        hdu.header['TSTOP'] = hdu.data['TIME'][-1] + dt
+        hdu.header['TFIRST'] = hdu.data['TIME'][0] - dt
+        hdu.header['TLAST'] = hdu.data['TIME'][-1] + dt
+        hdu.header['TELAPSE'] = hdu.header['TLAST'] - hdu.header['TFIRST']
+
+        ontime=0
+        for x in hdu.data['FRACEXP']:
+            ontime += x * timedel
+
+        hdu.header['ONTIME'] = ontime
+
         fits.writeto(lc_fn, hdu.data, header=hdu.header, overwrite=True)
 
         mjdref = float(hdu.header['MJDREF'])
@@ -923,44 +1064,45 @@ class INTEGRALwrapper(object):
         plt.xlim([ra.max(), ra.min()])
         plt.ylim([dec.min(), dec.max()])
 
-        ras = np.array([x for x in sources['ra']])
-        decs = np.array([x for x in sources['dec']])
-        names = np.array([x for x in sources['src_names']])
-        sigmas = np.array([x for x in sources['significance']])
-        # Defines relevant indexes for plotting regions
+        if len(sources) > 0:
+            ras = np.array([x for x in sources['ra']])
+            decs = np.array([x for x in sources['dec']])
+            names = np.array([x for x in sources['src_names']])
+            sigmas = np.array([x for x in sources['significance']])
+            # Defines relevant indexes for plotting regions
 
-        m_new = np.array(['NEW' in name for name in names])
+            m_new = np.array(['NEW' in name for name in names])
 
-        # m_offaxis = self.source_results.MEAN_VAR > 5
-        # m_noisy = self.source_results.DETSIG_CORR < 5
-        m_noisy = sigmas < 5
+            # m_offaxis = self.source_results.MEAN_VAR > 5
+            # m_noisy = self.source_results.DETSIG_CORR < 5
+            m_noisy = sigmas < 5
 
-        # plot new sources as pink circles
+            # plot new sources as pink circles
 
-        try:
-            m = m_new & (sigmas > det_sigma)
-            ra_coord = ras[m]
-            dec_coord = decs[m]
-            new_names = names[m]
-            if zero_crossing:
-                ind_ra = ra_coord > 180.
-                try:
-                    ra_coord[ind_ra] -= 360.
-                except:
-                    pass
-        except:
-            ra_coord = []
-            dec_coord = []
-            new_names = []
+            try:
+                m = m_new & (sigmas > det_sigma)
+                ra_coord = ras[m]
+                dec_coord = decs[m]
+                new_names = names[m]
+                if zero_crossing:
+                    ind_ra = ra_coord > 180.
+                    try:
+                        ra_coord[ind_ra] -= 360.
+                    except:
+                        pass
+            except:
+                ra_coord = []
+                dec_coord = []
+                new_names = []
 
-        plt.scatter(ra_coord, dec_coord, s=100, marker="o", facecolors='none',
-                    edgecolors='pink',
-                    lw=3, label="NEW any", zorder=5)
-        for i in range(len(ra_coord)):
-            # print("%f %f %s\n"%(ra_coord[i], dec_coord[i], names[i]))
-            plt.text(ra_coord[i],
-                     dec_coord[i] + 0.5,
-                     new_names[i], color="pink", size=15)
+            plt.scatter(ra_coord, dec_coord, s=100, marker="o", facecolors='none',
+                        edgecolors='pink',
+                        lw=3, label="NEW any", zorder=5)
+            for i in range(len(ra_coord)):
+                # print("%f %f %s\n"%(ra_coord[i], dec_coord[i], names[i]))
+                plt.text(ra_coord[i],
+                         dec_coord[i] + 0.5,
+                         new_names[i], color="pink", size=15)
 
         # CF Plots object of interest as green
         for ooi in objects_of_interest:
@@ -976,15 +1118,15 @@ class INTEGRALwrapper(object):
             else:
                 raise Exception("fail")
 
-            INTEGRALwrapper.__log.debug("object:", ooi, c)
+            INTEGRALwrapper.__log.debug(f"object: {ooi} , {c}")
 
-            plt.scatter(c.ra.deg, c.dec.deg, marker="o", facecolors='none',
-                        edgecolors='white',
+            plt.scatter(c.ra.deg, c.dec.deg, marker="o", facecolors='none', s=100,
+                        edgecolors='green',
                         lw=3, label="Added", zorder=5)
             # for i in range(len(c)):
             plt.text(c.ra.deg,
                      c.dec.deg + 0.5,
-                     str(ooi), color="white", size=15)
+                     str(ooi), color="green", size=15)
 
         # m = m_new & m_offaxis
         # p.scatter(self.coord[m].ra.deg, self.coord[m].dec.deg, s=100, marker="o", facecolors='none', edgecolors='red',
@@ -1045,7 +1187,7 @@ class INTEGRALwrapper(object):
     def get_pointings(ra, dec, radius, tstart='2003-01-01T00:00:00', tstop='2020-04-01T00:00:00', type='cons',
                       min_good_isgri=500):
 
-        url = oda_public_host + '/gw/timesystem/api/v1.0/scwlist/' + type + '/'
+        url = legacy_oda_public_host + '/gw/timesystem/api/v1.0/scwlist/' + type + '/'
         url += tstart + '/' + tstop + '?'
         url += 'ra=%.4f&dec=%.4f&radius=%.2f&min_good_isgri=%.0f&return_columns=SWID,RA_SCX,DEC_SCX' % (
             ra, dec, radius, min_good_isgri)
@@ -1068,7 +1210,7 @@ class INTEGRALwrapper(object):
 
     @staticmethod
     def get_utc_from_revnum(revnum):
-        url = oda_public_host + '/gw/timesystem/api/v1.0/converttime/REVNUM/%04d/IJD' % revnum
+        url = legacy_oda_public_host + '/gw/timesystem/api/v1.0/converttime/REVNUM/%04d/IJD' % revnum
         ijd = requests.get(url).json()
         ijd_start = ijd.split()[1]
         ijd_stop = ijd.split()[2]
@@ -1081,7 +1223,7 @@ class INTEGRALwrapper(object):
     @staticmethod
     def converttime(informat, intime, outformat, debug=True):
         import time
-        url = oda_public_host + '/gw/timesystem/api/v1.0/converttime/' + \
+        url = legacy_oda_public_host + '/gw/timesystem/api/v1.0/converttime/' + \
               informat + '/' + t2str(intime) + '/' + outformat
         ntries_left = 3
 
